unit uUPCalendar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, Calendar;

type
  TUPCalendar = class(TCalendar)
  private
    FWeekendColor: TColor;
    FActiveDayColor: TColor;
    { Private declarations }
  protected
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
    procedure WMSize(var Message: TWMSize); override;
    procedure SetWeekendColor(Value: TColor);
    procedure SetActiveDayColor(Value: TColor);
    { Protected declarations }
  public
    constructor Create(aOwner: TComponent); override;
    { Public declarations }
  published
    property ColorWeekendDays: TColor read FWeekendColor write SetWeekendColor;
    property ColorActiveDay: TColor read FActiveDayColor write SetActiveDayColor;
    { Published declarations }
  end;

procedure Register;

implementation

constructor TUPCalendar.Create(aOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TUPCalendar.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);
var TheText: string;
begin
  DefaultDrawing := False;
  TheText := CellText[ACol, ARow];
  Canvas.Font.Style := Canvas.Font.Style - [fsBold];

  if (aCol = 0) or (aCol = 6)
  then Canvas.Font.Color := ColorWeekendDays
  else Canvas.Font.Color := Self.Font.Color;

  if (gdFocused in AState) or (gdSelected in AState)
  then with Canvas
       do begin
            Brush.Color := Color;
            FillRect(aRect);
            Pen.Color := clBtnHighLight;
            PolyLine([Point(aRect.Left, aRect.Bottom - 1), Point(aRect.Left, aRect.Top),
                     Point(aRect.Right - 1, aRect.Top)]);
            Pen.Color := clBtnShadow;
            PolyLine([Point(aRect.Right - 1, aRect.Top), Point(aRect.Right - 1, aRect.Bottom - 1),
                      Point(aRect.Left, aRect.Bottom - 1)]);
            InflateRect(aRect, -1, -1);
            Font.Color := ColorActiveDay;
            Font.Style := Font.Style + [fsBold];
            with ARect
            do TextRect(ARect, Left + (Right - Left - TextWidth(TheText)) div 2,
                        Top + (Bottom - Top - TextHeight(TheText)) div 2, TheText);
          end
  else with Canvas
       do begin
            FillRect(aRect);
            if (aRow = 0) and Ctl3D
            then begin
                   Pen.Color := clBtnHighLight;
                   PolyLine([Point(aRect.Left, aRect.Bottom - 1), Point(aRect.Left, aRect.Top),
                             Point(aRect.Right - 1, aRect.Top)]);
                   Pen.Color := clBtnShadow;
                   PolyLine([Point(aRect.Right - 1, aRect.Top), Point(aRect.Right - 1, aRect.Bottom - 1),
                             Point(aRect.Left, aRect.Bottom - 1)]);
                   InflateRect(aRect, -1, -1);
                 end;
//            else if (aRow = 0)
//                 then InflateRect(aRect, 0, -2);
            with ARect
            do TextRect(ARect, Left + (Right - Left - TextWidth(TheText)) div 2,
                        Top + (Bottom - Top - TextHeight(TheText)) div 2, TheText);
            if (aRow = 0) and not Ctl3D
            then begin
                   Pen.Color := clBtnShadow;
                   Canvas.MoveTo(1, aRect.Bottom - 2);
                   Canvas.LineTo(Width - 1, aRect.Bottom - 2);
                   Pen.Color := clBtnHighLight;
                   Canvas.MoveTo(Width - 1, aRect.Bottom - 1);
                   Canvas.LineTo(1, aRect.Bottom - 1);
//                   InflateRect(aRect, 0, -2);
                 end;
       end;

end;

procedure TUPCalendar.WMSize(var Message: TWMSize);
begin
  inherited WMSize(Message);
  DefaultRowHeight := (Message.Height - 6 * GridLineWidth - 4) div 7;
  RowHeights[0] := DefaultRowHeight + 4;
end;

procedure TUPCalendar.SetWeekendColor(Value: TColor);
begin
  FWeekendColor := Value;
  Invalidate;
end;

procedure TUPCalendar.SetActiveDayColor(Value: TColor);
begin
  FActiveDayColor := Value;
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('Upscene', [TUPCalendar]);
end;

end.
