unit uUPDateEdit;

interface

{$R *.dcr}
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
  Mask, fUPDateEdit, DB, DBCtrls;

const
  cDayLongWidth = 54;
  cDayShortWidth = 16;

type
  TDayDisplay = (ddLong, ddShort, ddNoDay);
  TDayLayout = (dlLeft, dlCenter, dlRight);

  TUPCustomDateEdit = class(TCustomMaskEdit)
  private
    ButtonWidth: Integer;
    ButtonPressed: Boolean;
    Calendar: TfrmCalendarEdit;

    FDateTime: TDateTime;
    FNoDateText: string;
    FDayDisplay: TDayDisplay;
    FDayLayout: TDayLayout;
    FFlat: Boolean;
    { Private declarations }
  protected
    procedure SetDate(Value: TDateTime); virtual;
    procedure SetDayDisplay(Value: TDayDisplay); virtual;
    procedure SetDayLayout(Value: TDayLayout); virtual;

    procedure CalendarDeActivate(Sender: TObject); virtual;
    procedure BoundsChanged; virtual;
    procedure DropDownClick(Value: Boolean); virtual;
    procedure SelectDate; virtual;
    procedure FormatText; virtual;

    procedure PaintWindow(DC: HDC); override;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMSetCursor(var Message: TWMSetCursor); message WM_SETCURSOR;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Change; override;

    property Date: TDateTime read FDateTime write SetDate;
    property NoDateText: string read FNoDateText write FNoDateText;

    property Font;
    property Text;
    property DayDisplay: TDayDisplay read FDayDisplay write SetDayDisplay;
    property DayLayout: TDayLayout read FDayLayout write SetDayLayout;
    property MonthButtonsFlat: Boolean read FFlat write FFlat;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure CreateParams(var Params: TCreateParams); override;

    { Public declarations }
  published
    { Published declarations }
  end;

  TUPDateEdit = class(TUPCustomDateEdit)
  private
  protected
  public
    property Date;
    property Text;
  published
    property DayDisplay;
    property DayLayout;
    property Enabled;
    property Font;
    property NoDateText;
    property ReadOnly;
    property TabOrder;
    property TabStop;
    property MonthButtonsFlat;
  end;

  TUPDBDateEdit = class(TUPCustomDateEdit)
  private
    FDataLink: TFieldDataLink;
    procedure InvokeEdit;
  protected
    function  GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    function  GetDataField: string;
    procedure SetDataField(Value: string);
    function  GetField: TField;
    procedure SetDate(Value: TDateTime); override;

    procedure DataChange(Sender: TObject);

    procedure Change; override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Field: TField read GetField;
    property Date;
    property Text;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DataField: string read GetDataField write SetDataField;
    property DayDisplay;
    property DayLayout;
    property Enabled;
    property Font;
    property NoDateText;
    property ReadOnly;
    property TabOrder;
    property TabStop;
    property MonthButtonsFlat;
  end;


procedure Register;

implementation

constructor TUPCustomDateEdit.Create(aOwner: TComponent);
begin
  inherited Create(AOwner);
  ButtonWidth := GetSystemMetrics(SM_CXVSCROLL);
  EditMask := '00-00-0000';
  FDateTime := SysUtils.Date;
  Width := 137;
  DayLayout := dlCenter;
//  FormatText;
end;

destructor TUPCustomDateEdit.Destroy;
begin
  inherited Destroy;
end;

procedure TUPCustomDateEdit.Loaded;
begin
  inherited Loaded;
  FormatText;
  BoundsChanged;
  Repaint;
end;

procedure TUPCustomDateEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or ES_MULTILINE;
end;

procedure TUPCustomDateEdit.PaintWindow(DC: HDC);
var r: TRect;
    Flags: Integer;
    day: string;
begin
  // draw day text using the current font

  Flags := 0;
  SelectObject(DC, Font.Handle);
  SetBkMode(DC, TRANSPARENT);
  r := Rect(1, 1, 1, 1);
  day := '';
  if Date <> 0
  then case DayDisplay of
         ddShort  : begin
                      day := FormatDateTime('ddd', Date);
                      r := Rect(1, 1, cDayShortWidth, Height);
                    end;
         ddLong   : begin
                      day := FormatDateTime('dddd', Date);
                      r := Rect(1, 1, cDayLongWidth, Height);
                    end;
       end;
  case DayLayout of
    dlLeft   : Flags := DT_LEFT;
    dlCenter : Flags := DT_CENTER;
    dlRight  : Flags := DT_RIGHT;
  end;
//  DrawText(DC, PChar(day), Length(day), r, DT_CALCRECT or DT_LEFT or DT_VCENTER);
  DrawText(DC, PChar(day), Length(day), r, Flags or DT_VCENTER);

  // draw drop down button
  Flags := 0;
  if not Enabled
  then Flags := Flags or DFCS_INACTIVE;
  if ButtonPressed
  then Flags := Flags or DFCS_FLAT or DFCS_PUSHED;

  SetRect(r, Width - Height + 1, 0, Width - 4, Height - 4);
  DrawFrameControl(DC, r, DFC_SCROLL, DFCS_SCROLLCOMBOBOX or Flags);
  ExcludeClipRect(DC, Width - 4 - ButtonWidth, 0, Width - 2, Height - 4);

  inherited PaintWindow(DC);
end;

procedure TUPCustomDateEdit.WMSetCursor(var Message: TWMSetCursor);
var p: TPoint;
    b: Boolean;
    w: Integer;
begin
  GetCursorPos(p);
  case DayDisplay of
    ddLong  : w := cDayLongWidth;
    ddShort : w := cDayShortWidth;
    ddNoDay : w := 0;
  end;
  b := (PtInRect(Rect(Width - 21, 0, Width, Height), ScreenToClient(p)));
  b := b or (PtInRect(Rect(0, 0, w, Height), ScreenToClient(p)));
  if b
  then Windows.SetCursor(LoadCursor(0, idc_Arrow))
  else inherited;
end;

procedure TUPCustomDateEdit.WMPaint(var Message: TWMPaint);
begin
  PaintHandler(Message);
end;

procedure TUPCustomDateEdit.WMSize(var Message: TWMSize);
begin
  inherited;
  BoundsChanged;
end;

procedure TUPCustomDateEdit.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var r: TRect;
    b: Boolean;
begin
  r := Rect(Width - Height, 0, Width, Height);
  b := (Button = mbLeft) and PtInRect(r, Point(x, y));
  if b
  then DropDownClick(b);
  InvalidateRect(Handle, @R, False);
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TUPCustomDateEdit.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var r: TRect;
begin
  ButtonPressed := not (Button = mbLeft);
  r := Rect(Width - Height, 0, Width, Height);
  InvalidateRect(Handle, @r, False);
  inherited MouseUp(Button, Shift, X, Y);
end;

procedure TUPCustomDateEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
end;

procedure TUPCustomDateEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (Chr(Key) in ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '/'])
     and (Date = 0)
  then begin
         Date := SysUtils.Date;
       end;
  case Key of
    VK_DELETE   : begin
                    Date := 0;
                    Key := 0;
                  end;
  end;
  inherited KeyDown(Key, Shift);
end;

procedure TUPCustomDateEdit.Change;
begin
  inherited Change;
  try
    if Text <> '  -  -    '
   // then FDateTime := StrToDateTime(Text);
    then Date := StrToDateTime(Text);
    Color := clWindow;
  except
    on EConvertError
    do Color := clRed;
  end;
  if Date = 0
  then Color := clWindow;
end;

procedure TUPCustomDateEdit.DropDownClick(Value: Boolean);
var h: HWND;
begin
  if Value and not ButtonPressed
  then ButtonPressed := True
  else ButtonPressed := False;

  if not ButtonPressed or ReadOnly
  then Exit;
  SelectDate;
end;

procedure TUPCustomDateEdit.BoundsChanged;
var r: TRect;
begin
  case DayDisplay of
    ddNoDay   : SetRect(r, 2, 0, Width - 6 - ButtonWidth, Height);
    ddLong    : SetRect(r, cDayLongWidth, 0, Width - 6 - ButtonWidth, Height);
    ddShort   : SetRect(r, cDayShortWidth, 0, Width - 6 - ButtonWidth, Height);
  end;
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@r));
  SendMessage(Handle, EM_SCROLLCARET, 0, 0);
end;

procedure TUPCustomDateEdit.SelectDate;
var aPoint: TPoint;
begin
  Calendar := TfrmCalendarEdit.Create(Self);
  try
    Calendar.btnLeft.Flat := FFlat;
    Calendar.btnRight.Flat := FFlat;
    aPoint := ClientOrigin;
    Calendar.Left := aPoint.x + Width - Calendar.Width - 2;
    Calendar.Top := aPoint.y + Height - 2;
//    Calendar.btnPrevious.Flat := FCalBtnsFlat;
//    Calendar.btnNext.Flat := FCalBtnsFlat;
    if FDateTime <> 0
    then Calendar.Calendar.CalendarDate := FDateTime
    else Calendar.Calendar.CalendarDate := SysUtils.Date;
    Calendar.Calendar.Invalidate;
    Calendar.OnDeactivate := CalendarDeactivate;
{    if Assigned(FOnDropDown)
    then FOnDropDown(Self);
}    Calendar.Show;
  except
    on Exception
    do raise Exception.Create('Unable to create Calendar for ' + Name);
  end;
end;

procedure TUPCustomDateEdit.CalendarDeactivate(Sender: TObject);
begin
  if (Sender as TForm).ModalResult = mrOK
  then begin
         Date := (Sender as TfrmCalendarEdit).Calendar.CalendarDate;
    //     if Assigned(FOnDateChanged)
    //     then FOnDateChanged(Self);
       end;
  (Sender as TForm).Close;
end;

procedure TUPCustomDateEdit.SetDate(Value: TDateTime);
begin
  if Value <> FDateTime
  then begin
         FDateTime := Value;
         FormatText;
         Repaint;
       end;
end;

procedure TUPCustomDateEdit.SetDayDisplay(Value: TDayDisplay);
begin
  if Value <> FDayDisplay
  then begin
         FDayDisplay := Value;
         BoundsChanged;
         Repaint;
       end;
end;

procedure TUPCustomDateEdit.SetDayLayout(Value: TDayLayout);
begin
  if Value <> FDayLayout
  then begin
         FDayLayout := Value;
         Repaint;
       end;
end;

procedure TUPCustomDateEdit.FormatText;
begin
  EditMask := '00-00-0000';
  if FDateTime = 0
  then begin
         EditMask := '';
         Text := NoDateText;
       end
  else Text := FormatDateTime('dd-mm-yyyy', FDateTime);
end;

constructor TUPDBDateEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
end;

destructor TUPDBDateEdit.Destroy;
begin
  FDataLink.Free;
  inherited Destroy;
end;

procedure TUPDBDateEdit.Change;
begin
  inherited Change;
end;

procedure TUPDBDateEdit.SetDate(Value: TDateTime);
begin
  if Assigned(Field) and not (csDesigning in ComponentState) and
     (FDataLink.DataSet.State <> dsInActive)
  then begin
         if Value <> FDateTime
         then InvokeEdit;
         if (FDataLink.DataSet.State <> dsBrowse)
         then begin
                Field.AsDateTime := Value;
                inherited SetDate(Value);
              end
       end
  else inherited SetDate(Value);
end;

function TUPDBDateEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TUPDBDateEdit.SetDataSource(Value: TDataSource);
begin
  if Value <> FDataLink.DataSource
  then FDataLink.DataSource := Value;
end;

function TUPDBDateEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TUPDBDateEdit.SetDataField(Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TUPDBDateEdit.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TUPDBDateEdit.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil
  then begin
         FDateTime := FDataLink.Field.AsDateTime;
         FormatText;
       end
  else if csDesigning in ComponentState
       then Text := Name
       else Text := '';
end;

procedure TUPDBDateEdit.InvokeEdit;
begin
  if Assigned(FDataLink) and DataSource.AutoEdit
  then FDataLink.Edit;
end;

procedure Register;
begin
  RegisterComponents('Upscene', [TUPDateEdit, TUPDBDateEdit]);
end;

end.
