unit uUPScrollbarButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TButtonKind = (bkLeft, bkUp, bkRight, bkDown);
  TUPScrollbarButton = class(TGraphicControl)
  private
    FKind: TButtonKind;
    FFlat: Boolean;
    MouseIsDown: Boolean;
    MouseIsIn: Boolean;
    { Private declarations }
  protected
    procedure SetKind(Value: TButtonKind);
    procedure SetFlat(Value: Boolean);
    function MouseIsDownAndInControl: Boolean;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure CMMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
    { Public declarations }
  published
    property Enabled;
    property Kind: TButtonKind read FKind write SetKind;
    property Flat: Boolean     read FFlat write SetFlat;

    property OnClick;
    property OnDblClick;
    { Published declarations }
  end;

procedure Register;

implementation

constructor TUPScrollbarButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 17;
  Height := 17;     
end;

procedure TUPScrollbarButton.Paint;
var btnType: Integer;
begin
  inherited Paint;
  case Kind of
    bkLeft   : btnType := DFCS_SCROLLLEFT;
    bkUp     : btnType := DFCS_SCROLLUP;
    bkRight  : btnType := DFCS_SCROLLRIGHT;
    bkDown   : btnType := DFCS_SCROLLDOWN;
  end;

  if MouseIsDownAndInControl
  then btnType := btnType or DFCS_PUSHED
  else if not MouseIsIn
       then btnType := btnType or DFCS_FLAT;
  if not Enabled
  then btnType := btnType or DFCS_INACTIVE;
  DrawFrameControl(Canvas.Handle, GetClientRect, DFC_SCROLL, btnType);
end;

procedure TUPScrollbarButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  MouseIsDown := True;
  Repaint;
end;

procedure TUPScrollbarButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  MouseIsDown := False;
  Repaint;
end;

procedure TUPScrollbarButton.CMMouseEnter(var Msg: TMessage);
begin
  inherited;
  MouseIsIn := True;
  Repaint;
end;

procedure TUPScrollbarButton.CMMouseLeave(var Msg: TMessage);
begin
  inherited;
  MouseIsIn := False;
  Repaint;
end;

procedure TUPScrollbarButton.SetKind(Value: TButtonKind);
begin
  FKind := Value;
  Invalidate;
end;

procedure TUPScrollbarButton.SetFlat(Value: Boolean);
begin
  if Value <> FFlat
  then begin
         FFlat := Value;
         Repaint;
       end;
end;

function TUPScrollbarButton.MouseIsDownAndInControl: Boolean;
var p: TPoint;
begin
  {GetCursorPos(p);
  Result := PtInRect(ClientRect, ScreenToClient(p));}
  Result := MouseIsIn and MouseIsDown;
end;

procedure Register;
begin
//  RegisterComponents('Upscene', [TUPScrollbarButton]);
end;

end.
